package com.googlecode.mad.mvntools.settingsutil;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * @goal generate
 */
public class M2EclipseGenerator extends AbstractMojo {
	
	/**
	 * * The project dependencies. *
	 * 
	 * @parameter expression="${project.build.directory}"
	 */
	private String		buildDir;
	
	private String[]	allresources	= {

										"src/main/resources", "src/test/resources", "src/main/jasperreports" };
	
	private String[]	allsources		= { "src/main/java", "src/main/java/src-gen", "src/main/java/src-domain", "src/main/java/src", "ejbModule", "connectorModule", "src", "src/test/java",
			"gen/src", "test"			};
	
	public void execute() throws MojoExecutionException {
		
		getLog().info("**********************************************************");
		getLog().info("*** m2 eclipse generator: " + MavenVersionUtil.getInstance().getVersionString() + "     ***");
		getLog().info("**********************************************************");
		getLog().info("");
		
		StringBuilder b = new StringBuilder();
		
		String rootpath = this.buildDir.replace("target", "");
		
		String classpath = "";
		
		// hinzufuegen der sourcen, die als solche existieren
		for (String elem : allsources) {
			File temp = new File(rootpath + elem + "/com");
			if (temp.exists()) {
				b.append("	<classpathentry kind=\"src\" path=\"" + elem + "\"/>\n");
			}
		}
		
		for (String elem : allresources) {
			File temp = new File(rootpath + elem);
			if (temp.exists()) {
				b.append("	<classpathentry kind=\"src\" path=\"" + elem + "\"/>\n");
			}
		}
		
		if ("".equals(b.toString())) {
			b.append("	<classpathentry kind=\"src\" path=\"src\"/>\n");
		}
		
		classpath = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + "<classpath>\n" + b.toString()
				+ "	<classpathentry kind=\"con\" path=\"org.eclipse.jdt.launching.JRE_CONTAINER/org.eclipse.jdt.internal.debug.ui.launcher.StandardVMType/J2SE-1.5\"/>\n"
				+ "	<classpathentry kind=\"con\" path=\"org.maven.ide.eclipse.MAVEN2_CLASSPATH_CONTAINER\"/>\n" + "	<classpathentry kind=\"output\" path=\"target/classes\"/>\n" + "</classpath>\n";
		
		// fix, falls keine path gesetzt wurde
		classpath = classpath.replace("path=\"\"", "path=\"src\"");
		
		String project = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + "<projectDescription>\n" + "	<name>mojo</name>\n" + "	<comment></comment>\n" + "	<projects>\n" + "	</projects>\n"
				+ "	<buildSpec>\n" + "		<buildCommand>\n" + "			<name>org.eclipse.jdt.core.javabuilder</name>\n" + "			<arguments>\n" + "			</arguments>\n" + "		</buildCommand>\n"
				+ "		<buildCommand>\n" + "			<name>org.maven.ide.eclipse.maven2Builder</name>\n" + "			<arguments>\n" + "			</arguments>\n" + "		</buildCommand>\n" + "	</buildSpec>\n"
				+ "	<natures>\n" + "		<nature>org.maven.ide.eclipse.maven2Nature</nature>\n" + "		<nature>org.eclipse.jdt.core.javanature</nature>\n" + "	</natures>\n" + "</projectDescription>\n";
		
		String m2eclipsesettings = "#Gesetzt durch eclipsegenerator2 version: " + MavenVersionUtil.getInstance().getVersionString() + "  Wed May 26 17:30:08 CEST 2010\n" + "activeProfiles=\n"
				+ "eclipse.preferences.version=1\n" + "fullBuildGoals=process-test-resources\n" + "includeModules=false\n" + "resolveWorkspaceProjects=false\n"
				+ "resourceFilterGoals=process-resources resources\\:testResources\n" + "skipCompilerPlugin=true\n" + "version=1";
		
		// .settings order erstellen, wenn nicht vorhanden
		File settings = new File(this.buildDir.replace("target", "") + "/.settings");
		
		if (!settings.exists()) {
			settings.mkdir();
		}
		
		// Files erstellen
		File m2eclipse = new File(rootpath + "/.settings/org.maven.ide.eclipse.prefs");
		
		File pr = new File(rootpath + ".project");
		File cl = new File(rootpath + ".classpath");
		
		// loeschen und neue erstellen der files
		if (m2eclipse.exists()) {
			m2eclipse.delete();
		}
		
		try {
			m2eclipse.createNewFile();
		} catch (IOException e4) {
			e4.printStackTrace();
		}
		
		if (cl.exists()) {
			cl.delete();
		}
		
		try {
			cl.createNewFile();
		} catch (IOException e4) {
			e4.printStackTrace();
		}
		
		if (pr.exists()) {
			pr.delete();
			
		}
		
		try {
			pr.createNewFile();
		} catch (IOException e4) {
			e4.printStackTrace();
		}
		
		// inhalte schreiben
		
		try {
			setContents(m2eclipse, m2eclipsesettings);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		try {
			setContents(pr, project);
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			setContents(cl, classpath);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		// loeschen der files in .settings
		File f1 = new File(rootpath + "/.settings/.webspheredeploy");
		if (f1.exists()) {
			f1.delete();
		}
		File f2 = new File(rootpath + "/.settings/com.ibm.etools.ejbdeploy.prefs");
		if (f2.exists()) {
			f2.delete();
		}
		
		File f4 = new File(rootpath + "/.settings/org.eclipse.jst.common.project.facet.core.prefs");
		if (f4.exists()) {
			f4.delete();
		}
		
		File f5 = new File(rootpath + "/.settings/org.eclipse.wst.common.component");
		if (f5.exists()) {
			f5.delete();
		}
		
		File f6 = new File(rootpath + "/.settings/org.eclipse.wst.common.project.facet.core.xml");
		if (f6.exists()) {
			f6.delete();
		}
		
		File f7 = new File(rootpath + "/.settings/org.hibernate.eclipse.console.prefs");
		if (f7.exists()) {
			f7.delete();
		}
		
	}
	
	private void setContents(File aFile, String aContents) throws FileNotFoundException, IOException {
		if (aFile == null) {
			throw new IllegalArgumentException("File should not be null.");
		}
		if (!aFile.exists()) {
			throw new FileNotFoundException("File does not exist: " + aFile);
		}
		if (!aFile.isFile()) {
			throw new IllegalArgumentException("Should not be a directory: " + aFile);
		}
		if (!aFile.canWrite()) {
			throw new IllegalArgumentException("File cannot be written: " + aFile);
		}
		
		// use buffering
		Writer output = new BufferedWriter(new FileWriter(aFile));
		try {
			output.write(aContents);
		} finally {
			output.close();
		}
	}
}
