package com.googlecode.mad.mvntools.settingsutil;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * Dieses Singleton parst das File pom.properties und <br>
 * stellt Version und Buildatum zur Verfuegung <br>
 * <br>
 * LastChangedBy: $Author: marthaler.worb@gmail.com $<br>
 * <br>
 * LastChangedDate: $Date: 2010-12-21 20:39:28 +0100 (Di, 21 Dez 2010) $
 * 
 * @author (daniel marthaler)
 * @version $Revision: 17 $
 * @since 10.09.2010
 */
public class MavenVersionUtil {
	
	private static MavenVersionUtil	INSTANCE		= null;
	
	private Date					compileadet		= new Date(0L);
	
	private String					dateString		= "";
	
	private String					version			= "?";
	
	private final String			sourcepattern	= "#EEE MMM dd HH:mm:ss z yyyy";
	
	private final String			targetpattern	= "dd.MM.yy HH:mm";
	
	private final String			file			= "META-INF/maven/com.googlecode.mad-mvntools/settingsutil/pom.properties";
	
	private MavenVersionUtil() {
		
	}
	
	public static synchronized MavenVersionUtil getInstance() {
		if (MavenVersionUtil.INSTANCE == null) {
			MavenVersionUtil.INSTANCE = new MavenVersionUtil();
			MavenVersionUtil.INSTANCE.init();
		}
		return MavenVersionUtil.INSTANCE;
	}
	
	/**
	 * @return Datum als String, im Format, welches hier: targetpattern konfigurierbar ist
	 */
	public String getDateString() {
		if ("".equals(this.dateString)) {
			final SimpleDateFormat fm = new SimpleDateFormat(this.targetpattern);
			this.dateString = fm.format(this.compileadet);
		}
		return this.dateString;
	}
	
	/**
	 * @return aktuelle Maven Version
	 */
	public String getVersionString() {
		return this.version;
	}
	
	private void init() {
		final URL url = MavenVersionUtil.class.getClassLoader().getResource(this.file);
		
		if (url == null) {
			
			return;
		}
		
		InputStream oin = null;
		try {
			oin = new BufferedInputStream(url.openStream());
		} catch (final IOException e1) {
			
		}
		
		String line;
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(oin, "UTF-8"));
		} catch (final UnsupportedEncodingException e) {
			
		}
		try {
			while ((line = reader.readLine()) != null) {
				
				if (line.contains("CEST") || line.contains("CET")) {
					final SimpleDateFormat format = new SimpleDateFormat(this.sourcepattern, Locale.US);
					try {
						this.compileadet = format.parse(line);
					} catch (final ParseException e) {
						
					}
					
				}
				
				if (line.contains("version")) {
					this.version = line.replace("version=", "");
				}
			}
		} catch (final IOException e) {
			
		}
	}
}
