package com.googlecode.mad.mvntools.settingsutil.filestore;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.googlecode.mad.mvntools.settingsutil.MavenVersionUtil;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Abstract Mojo to backup and restore a .x files from a project
 */
public abstract class AbstractFilestore extends AbstractMojo {
	
	protected final static String	FOLDER	= "XXfolder:";
	protected final static String	FILE	= "XXfile:";
	
	protected String				name	= "abstractFilestore";
	
	protected XStream				xstream	= new XStream(new DomDriver());
	
	/**
	 * The project dependencies. *
	 * 
	 * @parameter expression="${project.build.directory} "
	 */
	protected String				buildDir;
	
	protected String[]				files	= { ".project", ".classpath" };
	
	protected String[]				folders	= { ".settings" };
	
	/**
	 * The configured files 
	 * 
	 * @parameter alias="files"
	 */
	private String[]				providedFiles;
	
	/**
	 * The configured folders 
	 * 
	 * @parameter alias="folders"
	 */
	private String[]				providedFolders;
	
	/**
	 * The name of the backup-file
	 * 
	 * @parameter alias="backupfile" default-value="settingsutil.xml"
	 */
	protected String				backupFile;
	
	/**
	 * The User
	 * 
	 * @parameter alias="user" default-value="default"
	 */
	protected String				user;
	
	public void execute() throws MojoExecutionException {
		
		String first = "************************************************************************";
		
		StringBuilder b = new StringBuilder();
		b.append("*** ");
		b.append(name);
		b.append(": ");
		b.append(MavenVersionUtil.getInstance().getVersionString());
		b.append(", built on ");
		b.append(MavenVersionUtil.getInstance().getDateString());
		
		while ((b.length() + 4) < first.length()) {
			b.append(" ");
		}
		b.append(" ***");
		
		getLog().info(first);
		getLog().info(b.toString());
		getLog().info(first);
		getLog().info("");
		
		// overwrite the default files and folders if there are cnfigured ones
		if (providedFiles != null && providedFiles.length > 0) {
			this.files = providedFiles;
		}
		
		if (providedFolders != null && providedFolders.length > 0) {
			this.folders = providedFolders;
		}
		
		myExecute();
		
	}
	
	protected abstract void myExecute();
	
	protected String getRootPath() {
		return this.buildDir.replace("target", "");
	}
	
}
