package com.googlecode.mad.mvntools.settingsutil.utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Writer;
import java.net.URL;

/**
 * LastChangedBy: $Author: marthaler.worb@gmail.com $<br>
 * LastChangedDate: $Date: 2010-12-21 20:40:35 +0100 (Di, 21 Dez 2010) $
 * 
 * @author  daniel marthaler
 * @version $Revision: 18 $
 * @since 26.06.2010
 */
public final class FileUtil {
	
	private static FileUtil	INSTANCE	= null;
	
	private FileUtil() {
		
	}
	
	public static synchronized FileUtil getInstance() {
		if (FileUtil.INSTANCE == null) {
			FileUtil.INSTANCE = new FileUtil();
		}
		return FileUtil.INSTANCE;
	}
	
	public String loadFile(String name) {
		StringBuilder b = new StringBuilder();
		
		final URL url = FileUtil.class.getClassLoader().getResource(name);
		BufferedReader in = null;
		if (url == null) {
			return loadFile2(name);
		}
		try {
			in = new BufferedReader(new InputStreamReader(url.openStream()));
			String inputLine;
			
			while ((inputLine = in.readLine()) != null) {
				b.append(inputLine);
			}
			
			in.close();
		} catch (IOException e) {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e1) {
					
				}
			}
		}
		return b.toString();
	}
	
	private String loadFile2(String name) {
		StringBuilder b = new StringBuilder();
		FileReader r = null;
		try {
			r = new FileReader(name);
		} catch (FileNotFoundException e2) {
			return "";
		}
		
		BufferedReader in = null;
		
		try {
			in = new BufferedReader(r);
			String inputLine;
			
			while ((inputLine = in.readLine()) != null) {
				b.append(inputLine);
			}
			
			in.close();
		} catch (IOException e) {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e1) {
					return "";
				}
			}
		}
		return b.toString();
	}
	
	private void setContents(File aFile, String aContents) throws FileNotFoundException, IOException {
		if (aFile == null) {
			throw new IllegalArgumentException("File should not be null.");
		}
		if (!aFile.exists()) {
			throw new FileNotFoundException("File does not exist: " + aFile);
		}
		if (!aFile.isFile()) {
			throw new IllegalArgumentException("Should not be a directory: " + aFile);
		}
		if (!aFile.canWrite()) {
			throw new IllegalArgumentException("File cannot be written: " + aFile);
		}
		
		// use buffering
		Writer output = new BufferedWriter(new FileWriter(aFile));
		try {
			output.write(aContents);
		} finally {
			output.close();
		}
	}
	
	public void recreateFileWithNewContent(String file, String content) {
		File f = new File(file);
		if (f.exists()) {
			f.delete();
		}
		try {
			f.createNewFile();
		} catch (IOException e4) {
			e4.printStackTrace();
		}
		if (content != null && !"".equals(content)) {
			try {
				this.setContents(f, content);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	public void deleteFile(String file) {
		File f1 = new File(file);
		if (f1.exists()) {
			f1.delete();
		}
	}
}
